@@ -1,6 +1,6 @@
 /*
  * dhcpcd-dbus
- * Copyright 2009-2012 Roy Marples <roy@marples.name>
+ * Copyright 2009-2014 Roy Marples <roy@marples.name>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -63,7 +63,7 @@ append_config_value(DBusMessageIter *entry, int type,
 			retval = 0;
 		break;
 	case DBUS_TYPE_BYTE:
-		byte = strtoul(data, NULL, 0);
+		byte = (unsigned char)strtoul(data, NULL, 0);
 		dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
 		    DBUS_TYPE_BYTE_AS_STRING, &var);
 		if (dbus_message_iter_append_basic(&var, DBUS_TYPE_BYTE,
@@ -78,7 +78,7 @@ append_config_value(DBusMessageIter *entry, int type,
 			retval = 0;
 		break;
 	case DBUS_TYPE_INT16:
-		i16 = strtol(data, NULL, 0);
+		i16 = (dbus_int16_t)strtol(data, NULL, 0);
 		dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
 		    DBUS_TYPE_INT16_AS_STRING, &var);
 		if (dbus_message_iter_append_basic(&var,
@@ -86,7 +86,7 @@ append_config_value(DBusMessageIter *entry, int type,
 			retval = 0;
 		break;
 	case DBUS_TYPE_UINT16:
-		u16 = strtoul(data, NULL, 0);
+		u16 = (dbus_uint16_t)strtoul(data, NULL, 0);
 		dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
 		    DBUS_TYPE_UINT16_AS_STRING, &var);
 		if (dbus_message_iter_append_basic(&var,
@@ -94,7 +94,7 @@ append_config_value(DBusMessageIter *entry, int type,
 			retval = 0;
 		break;
 	case DBUS_TYPE_INT32:
-		i32 = strtol(data, NULL, 0);
+		i32 = (dbus_int32_t)strtol(data, NULL, 0);
 		dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
 		    DBUS_TYPE_INT32_AS_STRING, &var);
 		if (dbus_message_iter_append_basic(&var,
@@ -105,7 +105,7 @@ append_config_value(DBusMessageIter *entry, int type,
 		if (strchr(data, '.') != NULL && inet_aton(data, &in) == 1)
 			u32 = in.s_addr;
 		else
-			u32 = strtoul(data, NULL, 0);
+			u32 = (dbus_uint32_t)strtoul(data, NULL, 0);
 		dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
 		    DBUS_TYPE_UINT32_AS_STRING, &var);
 		if (dbus_message_iter_append_basic(&var,
@@ -168,7 +168,7 @@ append_config_array(DBusMessageIter *entry, int type, const char *data)
 			    inet_aton(tok, &in) == 1)
 				u32 = in.s_addr;
 			else
-				u32 = strtoul(tok, NULL, 0);
+				u32 = (dbus_uint32_t)strtoul(tok, NULL, 0);
 			ok = dbus_message_iter_append_basic(&array,
 			    DBUS_TYPE_UINT32, &u32);
                         break;
