@@ -1,6 +1,6 @@
 /*
  * dhcpcd-dbus
- * Copyright 2009-2012 Roy Marples <roy@marples.name>
+ * Copyright 2009-2014 Roy Marples <roy@marples.name>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -91,7 +91,7 @@ static const char dhcpcd_introspection_xml[] =
     "      <arg name=\"status\" type=\"s\"/>\n"
     "    </signal>\n";
 
-static const struct o_dbus const dhos[] = {
+static const struct o_dbus dhos[] = {
 	{ "interface=", DBUS_TYPE_STRING, 0, "Interface" },
 	{ "reason=", DBUS_TYPE_STRING, 0, "Reason" },
 	{ "if_up=", DBUS_TYPE_BOOLEAN, 0, "Up" },
@@ -226,11 +226,15 @@ static const struct o_dbus const dhos[] = {
 	{ "subnet_selection=", DBUS_TYPE_UINT32, 0, "SubnetSelection" },
 	{ "domain_search=", DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
 	  "DomainSearch" },
-	{ "ra1_prefix=", DBUS_TYPE_STRING, 0, "RA_Prefix" },
-	{ "ra1_prefix_len=", DBUS_TYPE_BYTE, 0, "RA_PrefixLen" },
+
+	{ "ra1_prefix=", DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, "RA_Prefix" },
 	{ "ra1_mtu=", DBUS_TYPE_UINT16, 0, "RA_MTU" },
 	{ "ra1_rdnss=", DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, "RA_RDNSS" },
 	{ "ra1_dnssl=", DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, "RA_DNSSL" },
+
+	{ "dhcp6_ia_na1_ia_addr1=", DBUS_TYPE_STRING, 0, "D6_IPAddress" },
+	{ "dhcp6_domain_search=",  DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+	    "D6_DomainSearch" },
 	{ NULL, 0, 0, NULL }
 };
 
@@ -311,6 +315,11 @@ dhcpcd_dbus_signal_status(const char *status)
 	DBusMessageIter args;
 
 	syslog(LOG_INFO, "status changed to %s", status);
+	if (connection == NULL) {
+		syslog(LOG_WARNING,
+		    "no DBus connection to notify of status change");
+		return;
+	}
 	msg = dbus_message_new_signal(DHCPCD_PATH, DHCPCD_SERVICE,
 	    "StatusChanged");
 	if (msg == NULL) {
@@ -333,6 +342,13 @@ dhcpcd_dbus_configure(const struct dhcpcd_config *c)
 	DBusMessageIter args, dict;
 	const char *reason;
 	struct o_dbus type;
+	const char *up;
+
+	if (connection == NULL) {
+		syslog(LOG_WARNING,
+		    "no DBus connection for dhcpcd event");
+		return;
+	}
 
 	msg = dbus_message_new_signal(DHCPCD_PATH, DHCPCD_SERVICE, "Event");
 	if (msg == NULL) {
@@ -349,9 +365,10 @@ dhcpcd_dbus_configure(const struct dhcpcd_config *c)
 	    DBUS_TYPE_VARIANT_AS_STRING
 	    DBUS_DICT_ENTRY_END_CHAR_AS_STRING,
 	    &dict);
+	up = dhcpcd_get_value(c, "if_up=");
 	if (strcmp(c->type, "ra") == 0) {
 		retval = append_config(&dict, NULL, c);
-	} else if (dhcpcd_get_value(c, "new_ip_address") ||
+	} else if ((up && strcmp(up, "true") == 0) ||
 	    strcmp(reason, "CARRIER") == 0)
 		retval = append_config(&dict, "new_", c);
 	else
@@ -550,7 +567,7 @@ dhcpcd_getconfig_blocks(DBusConnection *con, DBusMessage *msg)
 	if (blocks == NULL && errno)
 		return return_dbus_error(con, msg, S_EINVAL,
 		    "dhcpcd_list_blocks: %s", strerror(errno));
-	
+
 	reply = dbus_message_new_method_return(msg);
 	dbus_message_iter_init_append(reply, &args);
 	dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY,
@@ -595,7 +612,7 @@ dhcpcd_getconfig(DBusConnection *con, DBusMessage *msg)
 	if (opts == NULL && errno)
 		return return_dbus_error(con, msg, S_EINVAL,
 		    "dhcpcd_read_config: %s", strerror(errno));
-	
+
 	reply = dbus_message_new_method_return(msg);
 	dbus_message_iter_init_append(reply, &args);
 	dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY,
@@ -672,7 +689,7 @@ dhcpcd_setconfig(DBusConnection *con, DBusMessage *msg)
 			opt->value = NULL;
 		dbus_message_iter_next(&array);
 	}
-	dhcpcd_write_options(*block == '\0' ? NULL : block,
+	dhcpcd_write_options(block == NULL || *block == '\0' ? NULL : block,
 	    *name == '\0' ? NULL : name, opts);
 	while (opts) {
 		opt = opts->next;
@@ -688,6 +705,7 @@ dhcpcd_setconfig(DBusConnection *con, DBusMessage *msg)
 static DBusHandlerResult
 msg_handler(DBusConnection *con, DBusMessage *msg, _unused void *data)
 {
+
 	if (dbus_message_is_method_call(msg,
 		DBUS_INTERFACE_INTROSPECTABLE, "Introspect"))
 		return introspect(con, msg);
@@ -730,7 +748,7 @@ msg_handler(DBusConnection *con, DBusMessage *msg, _unused void *data)
 static void
 dbus_event(int revents, void *watch)
 {
-	int flags;
+	unsigned int flags;
 
 	flags = 0;
 	if (revents & POLLIN)
@@ -744,7 +762,7 @@ dbus_event(int revents, void *watch)
 	if (flags != 0)
 		dbus_watch_handle((DBusWatch *)watch, flags);
 
-	if (connection != NULL) {
+	if (connection) {
 		dbus_connection_ref(connection);
 		while (dbus_connection_dispatch(connection) ==
 		    DBUS_DISPATCH_DATA_REMAINS)
@@ -756,7 +774,9 @@ dbus_event(int revents, void *watch)
 static dbus_bool_t
 add_watch(DBusWatch *watch, _unused void *data)
 {
-	int fd, flags, eflags;
+	int fd;
+	unsigned int flags;
+	short eflags;
 
 	fd = dbus_watch_get_unix_fd(watch);
 	flags = dbus_watch_get_flags(watch);
@@ -790,24 +810,15 @@ dhcpcd_dbus_init(void)
 
 	dbus_error_init(&err);
 	connection = dbus_bus_get(DBUS_BUS_SYSTEM, &err);
-	if (connection == NULL) {
-		if (dbus_error_is_set(&err))
-			syslog(LOG_ERR, "%s", err.message);
-		else
-			syslog(LOG_ERR, "failed to get a dbus connection");
-		return -1;
-	}
-			
-	ret = dbus_bus_request_name(connection, DHCPCD_SERVICE,
-	    DBUS_NAME_FLAG_REPLACE_EXISTING, &err);
 	if (dbus_error_is_set(&err)) {
 		syslog(LOG_ERR, "%s", err.message);
 		return -1;
 	}
-	if (ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-		syslog(LOG_ERR, "dbus: not primary owner");
+	if (connection == NULL) {
+		syslog(LOG_ERR, "failed to get a dbus connection");
 		return -1;
 	}
+
 	if (!dbus_connection_set_watch_functions(connection,
 		add_watch, remove_watch, NULL, NULL, NULL))
 	{
@@ -820,12 +831,29 @@ dhcpcd_dbus_init(void)
 		syslog(LOG_ERR, "dbus: failed to register object path");
 		return -1;
 	}
+
+	/* We need to request our name last */
+	ret = dbus_bus_request_name(connection, DHCPCD_SERVICE,
+	    DBUS_NAME_FLAG_REPLACE_EXISTING, &err);
+	if (dbus_error_is_set(&err)) {
+		syslog(LOG_ERR, "%s", err.message);
+		return -1;
+	}
+	if (ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
+		syslog(LOG_ERR, "dbus: not primary owner");
+		return -1;
+	}
+
+	/* Process the queue as the watchers don't seem to pickup
+	 * the first call if we have just been activated */
+	dbus_event(0, NULL);
 	return 0;
 }
 
 void
 dhcpcd_dbus_close(void)
 {
+
 	if (connection) {
 		dbus_connection_unref(connection);
 		connection = NULL;
