@@ -1,6 +1,6 @@
 /*
  * dhcpcd-dbus
- * Copyright 2009-2012 Roy Marples <roy@marples.name>
+ * Copyright 2009-2014 Roy Marples <roy@marples.name>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -24,12 +24,16 @@
  * SUCH DAMAGE.
  */
 
+// For strverscmp(3)
+#define _GNU_SOURCE
+
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
 
 #include <errno.h>
 #include <fcntl.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,15 +46,24 @@
 #include "eloop.h"
 #include "wpa.h"
 
+#ifndef SUN_LEN
+#define SUN_LEN(su) \
+	(sizeof(*(su)) - sizeof((su)->sun_path) + strlen((su)->sun_path))
+#endif
+
 char *dhcpcd_version = NULL;
 const char *dhcpcd_status = NULL;
 struct dhcpcd_config *dhcpcd_configs = NULL;
+static bool terminate_commands = false;
 
 static char *order;
 static char *cffile;
-int command_fd = -1;
+static int command_fd = -1;
 static int listen_fd = -1;
 
+static const char * const dhcpcd_types[] =
+    { "link", "ipv4", "ra", "dhcp6", NULL };
+
 int
 set_nonblock(int fd)
 {
@@ -68,14 +81,6 @@ set_nonblock(int fd)
 static char *lbuf;
 static size_t lbuf_len;
 
-#ifdef DEBUG_MEMORY
-static void
-free_lbuf(void)
-{
-	free(lbuf);
-}
-#endif
-
 /* Handy routine to read very long lines in text files.
  * This means we read the whole line and avoid any nasty buffer overflows.
  * We strip leading space and avoid comment lines, making the code that calls
@@ -86,20 +91,10 @@ get_line(FILE * __restrict fp)
 {
 	char *p, *e, *nbuf;
 	size_t last;
-#ifdef DEBUG_MEMORY
-	static int setup;
-#endif
 
 	if (feof(fp))
 		return NULL;
 
-#ifdef DEBUG_MEMORY
-	if (setup == 0) {
-		setup = 1;
-		atexit(free_lbuf);
-	}
-#endif
-
 	last = 0;
 	do {
 		if (lbuf == NULL || last != 0) {
@@ -135,48 +130,63 @@ get_line(FILE * __restrict fp)
 }
 
 static ssize_t
-_dhcpcd_command(int fd, const char *cmd, char **buffer) 
+_dhcpcd_command(int fd, const char *cmd, char **buffer)
 {
-	ssize_t bytes, len;
+	size_t len;
+	ssize_t bytes;
 	char c[1024], *p;
+	char *nbuf;
 
 	/* each argument is NULL seperated.
 	 * We may need to send a space one day, so the API
 	 * in this function may need to be improved */
-	bytes = strlen(cmd) + 1;
+	len = strlen(cmd) + 1;
+	if (terminate_commands)
+		len++;
+	if (len > sizeof(c)) {
+		errno = ENOBUFS;
+		return -1;
+	}
 	strlcpy(c, cmd, sizeof(c));
 	p = c;
 	while ((p = strchr(p, ' ')) != NULL)
 		*p++ = '\0';
-	bytes = write(fd, c, bytes);
-	if (bytes == -1)
+	if (terminate_commands) {
+		c[len - 2] = '\n';
+		c[len - 1] = '\0';
+	} else
+		c[len - 1] = '\0';
+	if (write(fd, c, len) == -1)
 		return -1;
 	if (buffer == NULL)
 		return 0;
 
-	bytes = read(fd, c, sizeof(ssize_t));
+	bytes = read(fd, c, sizeof(size_t));
 	if (bytes == 0 || bytes == -1)
 		return bytes;
-	memcpy(&len, c, sizeof(ssize_t));
-	*buffer = malloc(len + 1);
-	if (*buffer == NULL)
+	memcpy(&len, c, sizeof(size_t));
+	nbuf = realloc(*buffer, len + 1);
+	if (nbuf == NULL)
 		return -1;
+	*buffer = nbuf;
 	bytes = read(fd, *buffer, len);
-	if (bytes != -1 && bytes < len)
+	if (bytes != -1 && (size_t)bytes < len)
 		*buffer[bytes] = '\0';
 	return bytes;
 }
 
 ssize_t
-dhcpcd_command(const char *cmd, char **buffer) 
+dhcpcd_command(const char *cmd, char **buffer)
 {
+
 	return _dhcpcd_command(command_fd, cmd, buffer);
 }
 
 static int
 dhcpcd_open(void)
 {
-	int fd, len;
+	int fd;
+	socklen_t len;
 	struct sockaddr_un sun;
 
 	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
@@ -184,7 +194,7 @@ dhcpcd_open(void)
 	memset(&sun, 0, sizeof(sun));
 	sun.sun_family = AF_UNIX;
 	strlcpy(sun.sun_path, DHCPCD_SOCKET, sizeof(sun.sun_path));
-	len = sizeof(sun.sun_family) + strlen(sun.sun_path) + 1;
+	len = (socklen_t)SUN_LEN(&sun);
 	if (connect(fd, (struct sockaddr *)&sun, len) == 0)
 		return fd;
 	close(fd);
@@ -210,6 +220,7 @@ _get_value(const char *data, size_t len, const char *var)
 const char *
 dhcpcd_get_value(const struct dhcpcd_config *c, const char *var)
 {
+
 	return _get_value(c->data, c->data_len, var);
 }
 
@@ -250,17 +261,17 @@ sort_configs(void)
 {
 	struct dhcpcd_config *c, *nc = NULL, *nl = NULL;
 	char *tmp, *p, *token;
-	static const char *types[] = { "ipv4", "ra", NULL };
 	int i;
 
 	if (order == NULL)
 		return;
+
 	tmp = p = strdup(order);
 	while ((token = strsep(&p, " "))) {
 		if (*token == '\0')
 			continue;
-		for (i = 0; i < 2; i++) {
-			c = dhcpcd_get_config(token, types[i]);
+		for (i = 0; dhcpcd_types[i]; i++) {
+			c = dhcpcd_get_config(token, dhcpcd_types[i]);
 			if (c == NULL)
 				continue;
 			if (c->next)
@@ -280,11 +291,6 @@ sort_configs(void)
 	}
 	free(tmp);
 	dhcpcd_configs = nc;
-
-	printf ("new order: ");
-	for (c = dhcpcd_configs; c; c = c->next)
-		printf ("%s(%s) ", c->iface, c->type);
-	printf ("\n");
 }
 
 static struct dhcpcd_config *
@@ -294,24 +300,43 @@ prepend_config(char *data, size_t len)
 	struct dhcpcd_config *c;
 
 	iface = _get_value(data, len, "interface=");
-	if (iface == NULL) {
+	if (iface == NULL || *iface == '\0') {
 		syslog(LOG_ERR, "dhcpcd: no interface in config");
 		return NULL;
 	}
+	type = _get_value(data, len, "ifclass=");
+	if (type && *type != '\0') {
+		syslog(LOG_DEBUG, "dhcpcd: %s: skipping ifclass %s",
+		    iface, type);
+		return NULL;
+	}
 	reason = _get_value(data, len, "reason=");
-	if (reason == NULL) {
+	if (reason == NULL || *reason == '\0') {
 		syslog(LOG_ERR, "dhcpcd: no reason in config");
 		return NULL;
 	}
-	if (strcmp(reason, "ROUTERADVERT") == 0)
+	if (strcmp(reason, "RECONFIGURE") == 0) {
+		syslog(LOG_WARNING, "no RECONFIGURE support as yet");
+		return NULL;
+	}
+	if (strcmp(reason, "PREINIT") == 0 ||
+	    strcmp(reason, "UNKNOWN") == 0 ||
+	    strcmp(reason, "CARRIER") == 0 ||
+	    strcmp(reason, "NOCARRIER") == 0 ||
+	    strcmp(reason, "DEPARTED") == 0 ||
+	    strcmp(reason, "STOPPED") == 0)
+		type = "link";
+	else if (strcmp(reason, "ROUTERADVERT") == 0)
 		type = "ra";
+	else if (reason[strlen(reason) - 1] == '6')
+		type = "dhcp6";
 	else
 		type = "ipv4";
 	c = dhcpcd_get_config(iface, type);
 	if (c == NULL) {
 		c = malloc(sizeof(*c));
 		if (c == NULL) {
-			syslog(LOG_ERR, "malloc: %m");
+			syslog(LOG_ERR, "%s: malloc: %m", __func__);
 			return NULL;
 		}
 		if (dhcpcd_configs)
@@ -332,28 +357,26 @@ prepend_config(char *data, size_t len)
 static struct dhcpcd_config *
 read_config(int fd)
 {
-	char sbuf[sizeof(ssize_t)], *rbuf;
-	ssize_t bytes, len;
+	char sbuf[sizeof(size_t)], *rbuf;
+	size_t len;
+	ssize_t bytes;
 	struct dhcpcd_config *c;
 
 	bytes = read(fd, sbuf, sizeof(sbuf));
-	if (bytes == 0 || bytes == -1) {
-		syslog(LOG_ERR, "lost connection to dhcpcd");
-		return NULL;
-	}
+	if (bytes == 0 || bytes == -1)
+		goto lost;
 	memcpy(&len, sbuf, sizeof(len));
 	rbuf = malloc(len + 1);
 	if (rbuf == NULL) {
-		syslog(LOG_ERR, "malloc: %m");
+		syslog(LOG_ERR, "%s: malloc: %m", __func__);
 		return NULL;
 	}
 	bytes = read(fd, rbuf, len);
 	if (bytes == 0 || bytes == -1) {
-		syslog(LOG_ERR, "lost connection to dhcpcd");
 		free(rbuf);
-		return NULL;
+		goto lost;
 	}
-	if (bytes != len) {
+	if ((size_t)bytes != len) {
 		free(rbuf);
 		syslog(LOG_ERR, "dhcpcd: failed to read buffer");
 		return NULL;
@@ -365,6 +388,12 @@ read_config(int fd)
 		return NULL;
 	}
 	return c;
+
+lost:
+	syslog(LOG_ERR, "lost connection to dhcpcd");
+	dhcpcd_close();
+	add_timeout_sec(1, dhcpcd_init, NULL);
+	return NULL;
 }
 
 static void
@@ -421,6 +450,7 @@ free_configs(void)
 void
 free_option_value(struct option_value *o)
 {
+
 	free(o->option);
 	free(o->value);
 	free(o);
@@ -574,11 +604,10 @@ _options(int action, const char *block, const char *name,
 		}
 		if (block == NULL) {
 			if (!skip)
-				fputc('\n', fp);			
+				fputc('\n', fp);
 			for (i = 0; i < buf_len; i++) {
 				fputs(buf[i], fp);
 				fputc('\n', fp);
-				skip = buf[i][0] == '\0' ? 1 : 0;
 			}
 		}
 	} else
@@ -606,6 +635,7 @@ exit:
 struct option_value *
 dhcpcd_read_options(const char *block, const char *name)
 {
+
 	return _options(ACT_READ, block, name, NULL, NULL);
 }
 
@@ -634,11 +664,37 @@ dhcpcd_list_blocks(const char *block)
 	return list;
 }
 
+#ifndef __GLIBC__
+/* Good enough for our needs */
+static int
+strverscmp(const char *s1, const char *s2)
+{
+        int s1maj, s1min, s1part;
+	int s2maj, s2min, s2part;
+	int r;
+
+	s1min = s1part = 0;
+        if (sscanf(s1, "%d.%d.%d", &s1maj, &s1min, &s1part) < 1)
+		return -1;
+	s2min = s2part = 0;
+	if (sscanf(s2, "%d.%d.%d", &s2maj, &s2min, &s2part) < 1)
+		return -1;
+	r = s1maj - s2maj;
+	if (r != 0)
+		return r;
+	r = s1min - s2min;
+	if (r != 0)
+		return r;
+	return s1part - s2part;
+}
+#endif
+
 void
 dhcpcd_init(_unused void *data)
 {
 	char cmd[128];
-	ssize_t nifs, bytes;
+	ssize_t bytes;
+	size_t nifs;
 	struct dhcpcd_config *c;
 	static int last_errno;
 	const char *ifo;
@@ -662,6 +718,8 @@ dhcpcd_init(_unused void *data)
 		syslog(LOG_ERR, "failed to get dhcpcd version");
 		exit(EXIT_FAILURE);
 	}
+	terminate_commands =
+	    strverscmp(dhcpcd_version, "6.4.1") >= 0 ? true : false;
 
 	if (dhcpcd_command("--getconfigfile", &cffile) <= 0) {
 		syslog(LOG_ERR, "failed to get dhcpcd config file");
@@ -679,19 +737,19 @@ dhcpcd_init(_unused void *data)
 
 	free_configs();
 	dhcpcd_command("--getinterfaces", NULL);
-	bytes = read(command_fd, cmd, sizeof(ssize_t));
-	if (bytes != sizeof(ssize_t))
+	bytes = read(command_fd, cmd, sizeof(nifs));
+	if (bytes != sizeof(nifs))
 		return;
-	memcpy(&nifs, cmd, sizeof(ssize_t));
+	memcpy(&nifs, cmd, sizeof(nifs));
 	for (; nifs > 0; nifs--) {
 		c = read_config(command_fd);
+		if (c == NULL)
+			return;
 		ifo = dhcpcd_get_value(c, "interface_order=");
 		if (ifo != NULL) {
 			free(order);
 			order = strdup(ifo);
 		}
-		if (c == NULL)
-			return;
 		syslog(LOG_INFO, "retrieved interface %s (%s)",
 		    c->iface, dhcpcd_get_value(c, "reason="));
 	}
@@ -717,13 +775,13 @@ dhcpcd_close(void)
 	free_configs();
 	free(cffile);
 	cffile = NULL;
-#ifdef DEBUG_MEMORY
 	free(dhcpcd_version);
 	dhcpcd_version = NULL;
 	free(order);
 	order = NULL;
-#endif
 	dhcpcd_status = "down";
 	dhcpcd_dbus_signal_status(dhcpcd_status);
+	free(lbuf);
+	lbuf = NULL;
 	return retval;
 }
