@@ -1,6 +1,6 @@
-/* 
+/*
  * dhcpcd-dbus
- * Copyright 2009 Roy Marples <roy@marples.name>
+ * Copyright 2009-2014 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -52,7 +52,7 @@ static struct timeval now;
 
 static struct event {
 	int fd;
-	int flags;
+	short flags;
 	void (*callback)(void *);
 	void (*callback_f)(int, void *);
 	void *arg;
@@ -140,7 +140,7 @@ get_monotonic(struct timeval *tp)
 
 static int
 _add_event_flags(int fd,
-    int flags,
+    short flags,
     void (*callback)(void *),
     void (*callback_f)(int, void *),
     void *arg)
@@ -184,12 +184,14 @@ _add_event_flags(int fd,
 int
 add_event(int fd, void (*callback)(void *), void *arg)
 {
+
 	return _add_event_flags(fd, 0, callback, NULL, arg);
 }
 
 int
-add_event_flags(int fd, int flags, void (*callback)(int, void *), void *arg)
+add_event_flags(int fd, short flags, void (*callback)(int, void *), void *arg)
 {
+
 	return _add_event_flags(fd, flags, NULL, callback, arg);
 }
 
@@ -264,7 +266,7 @@ add_timeout_tv(const struct timeval *when, void (*callback)(void *), void *arg)
 		t->next = timeouts;
 		timeouts = t;
 		return 0;
-	} 
+	}
 	for (tt = timeouts; tt->next; tt = tt->next)
 		if (timercmp(&t->when, &tt->next->when, <)) {
 			t->next = tt->next;
@@ -346,7 +348,6 @@ delete_timeout(void (*callback)(void *), void *arg)
 	return n;
 }
 
-#ifdef DEBUG_MEMORY
 /* Define this to free all malloced memory.
  * Normally we don't do this as the OS will do it for us at exit,
  * but it's handy for debugging other leaks in valgrind. */
@@ -378,7 +379,6 @@ cleanup(void)
 	}
 	free(fds);
 }
-#endif
 
 _noreturn void
 start_eloop(void)
@@ -389,9 +389,7 @@ start_eloop(void)
 	struct timeout *t;
 	struct timeval tv;
 
-#ifdef DEBUG_MEMORY
 	atexit(cleanup);
-#endif
 
 	for (;;) {
 		/* Run all timeouts first.
@@ -412,8 +410,8 @@ start_eloop(void)
 				(tv.tv_usec + 999) / 1000 > INT_MAX % 1000))
 				msecs = INT_MAX;
 			else
-				msecs = tv.tv_sec * 1000 +
-					(tv.tv_usec + 999) / 1000;
+				msecs = (int)(tv.tv_sec * 1000 +
+					(tv.tv_usec + 999) / 1000);
 		} else
 			/* No timeouts, so wait forever. */
 			msecs = -1;
@@ -431,7 +429,7 @@ start_eloop(void)
 			free(fds);
 			fds = malloc(sizeof(*fds) * nfds);
 			if (fds == NULL) {
-				syslog(LOG_ERR, "malloc: %m");
+				syslog(LOG_ERR, "%s: malloc: %m", __func__);
 				exit(EXIT_FAILURE);
 			}
 			fds_len = nfds;
