@@ -1,6 +1,6 @@
 /*
  * dhcpcd-dbus
- * Copyright 2009 Roy Marples <roy@marples.name>
+ * Copyright 2009-2014 Roy Marples <roy@marples.name>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,21 +36,21 @@
 #include "dhcpcd-dbus.h"
 #include "wpa.h"
 
-const char copyright[] = "Copyright (c) 2009-2012 Roy Marples";
+const char copyright[] = "Copyright (c) 2009-2014 Roy Marples";
 
 static void
 cleanup(void)
 {
+
 	dhcpcd_dbus_close();
 	wpa_close(NULL);
-#ifdef DEBUG_MEMORY
 	dhcpcd_close();
-#endif
 }
 
 static void
 handle_signal(int sig)
 {
+
 	if (sig) {
 		syslog(LOG_INFO, "Got signal %d, exiting", sig);
 		exit(EXIT_SUCCESS);
@@ -60,6 +60,7 @@ handle_signal(int sig)
 int
 main(void)
 {
+
 	openlog(PACKAGE, LOG_PERROR, LOG_DAEMON);
 	setlogmask(LOG_UPTO(LOG_INFO));
 	syslog(LOG_INFO, "starting " PACKAGE "-" VERSION);
@@ -71,9 +72,11 @@ main(void)
 	/* Ignore pipes */
 	signal(SIGPIPE, SIG_IGN);
 
+	dhcpcd_init(NULL);
 	if (dhcpcd_dbus_init() == -1)
 		exit(EXIT_FAILURE);
-	dhcpcd_init(NULL);
+
+	syslog(LOG_INFO, "init completed, waiting for events");
 	start_eloop();
 	exit(EXIT_SUCCESS);
 }
