@@ -1,6 +1,6 @@
 /*
  * dhcpcd-dbus
- * Copyright 2009-2012 Roy Marples <roy@marples.name>
+ * Copyright 2009-2014 Roy Marples <roy@marples.name>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,6 +41,11 @@
 #include "wpa.h"
 #include "wpa-dbus.h"
 
+#ifndef SUN_LEN
+#define SUN_LEN(su) \
+	(sizeof(*(su)) - sizeof((su)->sun_path) + strlen((su)->sun_path))
+#endif
+
 struct if_sock {
 	char *iface;
 	int cmd_fd;
@@ -58,7 +63,8 @@ static int
 _wpa_open(const char *iface, char **path)
 {
 	static int counter;
-	int fd, len;
+	int fd;
+	socklen_t len;
 	struct sockaddr_un sun;
 
 	if ((fd = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1)
@@ -68,7 +74,7 @@ _wpa_open(const char *iface, char **path)
 	snprintf(sun.sun_path, sizeof(sun.sun_path),
 	    "/tmp/" PACKAGE "-wpa-%d.%d", getpid(), counter++);
 	*path = strdup(sun.sun_path);
-	len = sizeof(sun.sun_family) + strlen(sun.sun_path) + 1;
+	len = (socklen_t)SUN_LEN(&sun);
 	if (bind(fd, (struct sockaddr *)&sun, len) == -1) {
 		close(fd);
 		unlink(*path);
@@ -78,7 +84,7 @@ _wpa_open(const char *iface, char **path)
 	}
 	snprintf(sun.sun_path, sizeof(sun.sun_path),
 	    WPA_CTRL_DIR "/%s", iface);
-	len = sizeof(sun.sun_family) + strlen(sun.sun_path) + 1;
+	len = (socklen_t)SUN_LEN(&sun);
 	if (connect(fd, (struct sockaddr *)&sun, len) == 0) {
 		set_nonblock(fd);
 		return fd;
@@ -103,7 +109,7 @@ find_if_sock(const char *iface)
 }
 
 static ssize_t
-_wpa_cmd(int fd, const char *cmd, char *buffer, ssize_t len)
+_wpa_cmd(int fd, const char *cmd, char *buffer, size_t len)
 {
 	int retval;
 	ssize_t bytes;
@@ -134,7 +140,7 @@ _wpa_cmd(int fd, const char *cmd, char *buffer, ssize_t len)
 }
 
 ssize_t
-wpa_cmd(const char *iface, const char *cmd, char *buffer, ssize_t len)
+wpa_cmd(const char *iface, const char *cmd, char *buffer, size_t len)
 {
 	struct if_sock *ifs;
 
@@ -235,15 +241,15 @@ wpa_close(const char *iface)
 	return retval;
 }
 
-static int
+static ssize_t
 read_event(const struct if_sock *ifs)
 {
 	char buffer[256], *p;
-	ssize_t bytes;
+	size_t bytes;
 
-	bytes = read(ifs->ctrl_fd, buffer, sizeof(buffer));
-	if (bytes == -1 || bytes == 0)
-		return -1;
+	bytes = (size_t)read(ifs->ctrl_fd, buffer, sizeof(buffer));
+	if ((ssize_t)bytes == -1 || bytes == 0)
+		return (ssize_t)bytes;
 	buffer[bytes] = '\0';
 	bytes = strlen(buffer);
 	if (buffer[bytes - 1] == ' ')
@@ -255,12 +261,13 @@ read_event(const struct if_sock *ifs)
 		}
 	if (strcmp(p, "CTRL-EVENT-SCAN-RESULTS") == 0)
 		wpa_dbus_signal_scan_results(ifs->iface);
-	return bytes;
+	return (ssize_t)bytes;
 }
 
 static void
 handle_wpa(void *ifs)
 {
+
 	read_event((const struct if_sock *)ifs);
 }
 
@@ -280,7 +287,7 @@ ping(void *arg)
 
 	syslog(LOG_ERR, "lost connection to wpa_supplicant on interface %s",
 	    ifs->iface);
-	c = dhcpcd_get_config(ifs->iface, "ipv4");
+	c = dhcpcd_get_config(ifs->iface, "link");
 	wpa_close(c->iface);
 	add_timeout_sec(1, wpa_init, (void *)UNCONST(c->iface));
 }
